import React, {
    useEffect,
    useMemo,
    useState
} from "react";
import moment from "moment";
import {
    Redirect,
    Route,
    Switch,
    useParams
} from "react-router-dom";

import Races from "./Races";
import Driver from "./Driver";
import Team from "./Team";
import {
    getCurrentPointsOfDriver
} from "../utility/champCalculator";

import teams2023 from "../data/2023/teams.json";
import races2023 from "../data/2023/races.json";
import points2023 from "../data/2023/points.json";

import teams2024 from "../data/2024/teams.json";
import races2024 from "../data/2024/races.json";
import points2024 from "../data/2024/points.json";

import teams2025 from "../data/2025/teams.json";
import races2025 from "../data/2025/races.json";
import points2025 from "../data/2025/points.json";
import results2025 from "../data/2025/results.json";

export function getData() {
    return {
        2023: {
            teams: teams2023,
            races: races2023,
            points: points2023,
            pointsMultiTeam: points2023,
        },
        2024: {
            teams: teams2024,
            races: races2024,
            points: points2024,
            pointsMultiTeam: combineDriverPoints2024(points2024),
        },
        2025: {
            teams: teams2025,
            races: races2025,
            points: points2025,
            pointsMultiTeam: calculateDriverPoints2025(points2025, races2025, teams2025, results2025),
            results: results2025,
        },
    }
}
function combineDriverPoints2024(points) {
    let combinedDrivers = {};
    Object.entries(points.drivers).forEach(([driver, points]) => {
        const splitKey = driver.split("-");
        if(combinedDrivers[splitKey[0]] === undefined) {
            combinedDrivers[splitKey[0]] = 0;
        }
        combinedDrivers[splitKey[0]] += points;
    });
    return {
        updatedUntil: points.updatedUntil,
        drivers: combinedDrivers,
    };
}

function calculateDriverPoints2025(points, races, teams, results) {
    const drivers = Object.values(teams).map((team) => {
        return team.drivers.map((driver) => {
            return driver.split("-")[0];
        });
    }).reduce((acc, val) => acc.concat(val), []);
    let driverPoints = {};
    drivers.forEach((driver) => {
        if(!driverPoints[driver]) {
            driverPoints[driver] = 0;
        }
        driverPoints[driver] += getCurrentPointsOfDriver(driver, points, races, results);
    });
    let driverPointsSorted = {};
    Object.entries(driverPoints).sort((a, b) => {
        return b[1] - a[1];
    }).forEach(([key, value]) => {
        driverPointsSorted[key] = value;
    });
    return {
        updatedUntil: results.length,
        drivers: driverPointsSorted,
    };
}

export const YearContext = React.createContext(null);

function YearRoute() {
    const {
        year
    } = useParams();
    const [teams, setTeams] = useState(undefined);
    const [races, setRaces] = useState(undefined);
    const [points, setPoints] = useState(undefined);
    const [pointsMultiTeam, setPointsMultiTeam] = useState(undefined);

    const selectedYear = useMemo(() => {
        const currentYear = moment().year();
        const parsedYear = year ? parseInt(year) : null;
        if(!parsedYear) {
            return currentYear;
        }
        return parsedYear;
    }, [year]);

    useEffect(() => {
        if (!year) {
            setTeams(null);
            setRaces(null);
            setPoints(null);
            setPointsMultiTeam(null);
        }
        const data = getData();
        setTeams(data[year].teams);
        setRaces(data[year].races);
        setPoints(data[year].points);
        setPointsMultiTeam(data[year].pointsMultiTeam);
    }, [year]);

    return (
        <YearContext.Provider value={{
            teams,
            races,
            points,
            pointsMultiTeam,
            year: selectedYear
        }}>
            <Switch>
                <Route
                    path={`/${selectedYear}/races`}
                    exact
                    component={ Races }
                />
                <Route
                    path={ [`/${selectedYear}/drivers/:driver`, `/${selectedYear}/drivers`] }
                    exact
                    component={ Driver }
                />
                <Route
                    path={ [`/${selectedYear}/teams/:team`, `/${selectedYear}/teams`] }
                    exact
                    component={ Team }
                />
                <Route path="/">
                    <Redirect to={`/${selectedYear}/races`}/>
                </Route>
            </Switch>
        </YearContext.Provider>
    )
}

export default React.memo(YearRoute);
