import {
    getData
} from "../pages/YearRoute";

export function getPointsAvailableForRace(race, points, firstPlace = true) {
    if(points?.system) {
        const pointsSystem = race.gp ? points.system.gp : points.system.sprint;
        return pointsSystem.points[firstPlace ? 0 : 1] + pointsSystem.fastestLap;
    }
    if(race.gp) {
        return 26 - (firstPlace ? 0 : 8);
    } else if(race.sprint) {
        return 8 - (firstPlace ? 0 : 1);
    }
}

export function getCurrentPointsOfDriver(driver, points, races, results) {
    if(!results) {
        return points.drivers[driver];
    }
    let pointCounter = 0;
    races.forEach((race) => {
        pointCounter += getPointsOfDriverForRace(driver, points, race, results) ?? 0;
    });
    return pointCounter;
}

function getPointsOfDriverForRace(driver, points, race, results) {
    const pointsSystem = race.gp
        ? points.system?.gp
        : points.system?.sprint;
    const result = results.find((result) => {
        return result.code === race.code;
    });
    if(!result) {
        return null;
    }
    const driverShort = driver.split("-")[0];
    const positionsNoTeam = result.positions.map((position) => {
        return position.split("-")[0];
    });
    const position = positionsNoTeam.indexOf(driverShort) + 1;
    if(position === -1) {
        return 0;
    }
    const positionPoints = pointsSystem.points.find((points, index) => {
        return index + 1 === position;
    });
    if(result.fastestLap === driverShort) {
        return positionPoints + pointsSystem.fastestLap;
    }
    return positionPoints;
}

export function getRacesLeft(year) {
    const data = getData()[year];
    const lastRaceIndex = data.pointsMultiTeam.updatedUntil;
    return data.races.filter((race, index) => {
        return index >= lastRaceIndex;
    });
}

export function getTotalPointsLeft(year, points) {
    const racesLeft = getRacesLeft(year);
    let totalPoints = 0;
    racesLeft.forEach((race) => {
        totalPoints += getPointsAvailableForRace(race, points);
    })
    return totalPoints;
}

export function getPointsAvailableFromRace(year, points, targetRace = null) {
    let pointsAvailable = getTotalPointsLeft(year);
    if(!targetRace) {
        return pointsAvailable;
    }
    const racesLeft = getRacesLeft(year);
    for(const race of racesLeft) {
        pointsAvailable -= getPointsAvailableForRace(race, points);
        if(race.name === targetRace.name) {
            break;
        }
    }
    return pointsAvailable;
}

export function getPointsAheadMax(year, points, driver1, driver2, targetRace) {
    const data = getData()[year];
    const racesLeft = getRacesLeft(year);
    let pointsDriver1 = data.pointsMultiTeam.drivers[driver1];
    let pointsDriver2 = data.pointsMultiTeam.drivers[driver2];
    for(const race of racesLeft) {
        pointsDriver1 += getPointsAvailableForRace(race, points, true);
        pointsDriver2 += getPointsAvailableForRace(race, points, false);
        if(targetRace && race.name === targetRace.name) {
            break;
        }
    }
    return pointsDriver1 - pointsDriver2;
}

export function getPointsAheadMin(year, points, driver1, driver2, targetRace) {
    const data = getData()[year];
    const racesLeft = getRacesLeft(year);
    let pointsDriver1 = data.pointsMultiTeam.drivers[driver1];
    let pointsDriver2 = data.pointsMultiTeam.drivers[driver2];
    for(const race of racesLeft) {
        pointsDriver1 += getPointsAvailableForRace(race, points, true);
        if(targetRace && race.name === targetRace.name) {
            break;
        }
    }
    return pointsDriver1 - pointsDriver2;
}

export function getChampionshipPosition(year, driver) {
    const data = getData()[year];
    const drivers = Object.entries(data.pointsMultiTeam.drivers);
    drivers.sort((a, b) => {
        return b[1] - a[1];
    })
    return drivers.findIndex((driverPoints) => {
        return driverPoints[0] === driver;
    }) + 1;
}

export function getTeamPoints(year, team) {
    const {
        teams,
        points,
        pointsMultiTeam,
        races,
        results,
    } = getData()[year];
    const teamData = teams[team];
    let teamPoints = 0;
    for(const driver of teamData.drivers) {
        teamPoints += getCurrentPointsOfDriver(
            driver,
            points.system ? points : pointsMultiTeam,
            races,
            results
        );
    }
    return teamPoints;
}
